'use client';

import { HelpIcon } from '@/components/icons';
import { CreateUpdateJob } from '@/components/jobs';
import { Loader } from '@/components/others/CommonComponents';
import { URL_SEARCH_PARAMS } from '@/components/utils/constants';
import {
	fetchEndpointData,
	updateResourceEndpointData,
} from '@/components/utils/server-functions';
import {
	JobStatusEnum,
	TJob,
	TJobTypes,
	TMinistry,
} from '@/components/utils/types';
import { useQueryCustomWrapper } from '@/utils';
import { usePathname, useRouter } from 'next/navigation';
import { useState } from 'react';
import { useQuery } from 'react-query';
import { toast } from 'react-toastify';
import 'react-toastify/dist/ReactToastify.css';

export default function ({ params }: { params: { id: string } }) {
	const {
		isLoading: isMinistryLoading,
		isError: isMinistryError,
		data: ministries,
	} = useQuery({
		queryFn: useQueryCustomWrapper<TMinistry[]>,
		queryKey: [
			`ministry-data`,
			{
				url: `ministries`,
				qFunc: fetchEndpointData,
			},
		],
	});

	const { id } = params;

	const {
		isLoading: isJobtypeLoading,
		isError: isJobTypeError,
		data: jobTypes,
	} = useQuery({
		queryFn: useQueryCustomWrapper<TJobTypes[]>,
		queryKey: [
			`jobtype-data`,
			{
				url: `job-types`,
				qFunc: fetchEndpointData,
			},
		],
	});

	const { isLoading, isError, data } = useQuery({
		queryFn: useQueryCustomWrapper<TJob>,
		queryKey: [
			`jobs-data-${id}`,
			{
				url: `jobs/${id}`,
				qFunc: fetchEndpointData,
				options: {
					sort: 'createdAt:desc',
					populate: {
						applications: {
							populate: {
								user: true,
							},
						},
						job_type: true,
						ministry: true,
						user: true,
						files: true,
					},
				},
			},
		],
		enabled: !!id,
	});

	const [errMsg, setErrMsg] = useState<string>();
	const [loading, setLoading] = useState(false);
	const pathname = usePathname();
	const router = useRouter();

	const handleCloseJob = () => {
		setLoading(true);

		updateResourceEndpointData({
			url: `jobs/${id}`,
			data: { status: JobStatusEnum.Closed },
		})
			.then(({ err, data }) => {
				if (err) {
					if (err.status === 400) setErrMsg(err.message);
					else if (err.status === 401)
						router.push(
							`/?${URL_SEARCH_PARAMS.redirect}=${encodeURIComponent(pathname)}`
						);
					else if (err.status === 403) {
						toast.error('Something went wrong', { position: 'top-right' });
						setErrMsg('Permission denied');
					} else {
						toast.error('Something went wrong', { position: 'top-right' });

						setErrMsg('Something went wrong');
					}
				} else {
					toast.success('The job was closed', { position: 'top-right' });
				}
			})
			.finally(() => setLoading(false));
	};

	return (
		<div className='w-[96%] p-[20px] mb-[100px] bg-white border py-[16px] mx-auto shadow-applicantBoxDetailsShadow rounded-[8px]'>
			{isMinistryLoading || isJobtypeLoading || isLoading ? (
				<div className='w-full h-full'>
					<Loader />
				</div>
			) : isJobTypeError || isMinistryError || isError ? (
				<div className='w-full h-full'>
					<Loader {...{ title: 'Error loading data' }} />
				</div>
			) : (
				<CreateUpdateJob
					{...{
						jobTypes: jobTypes?.data ?? [],
						ministries: ministries?.data ?? [],
						title: 'Edit Job',
						titleEnd: (
							<>
								<button
									className='rounded-[8px] px-[12px] py-[8px] border border-btn-orange shadow-btnBoxShadow font-semibold text-[14px] leading-[24px] text-dark-orange cursor-pointer selection:bg-inherit'
									role='button'
									{...{
										onClick: handleCloseJob,
										disabled:
											loading || data?.data.status === JobStatusEnum.Closed,
									}}>
									Close Job
								</button>

								<div className='selection:bg-inherit'>
									<HelpIcon
										{...{
											svgElementClassName: 'stroke-gray-body-text',
											applyToSvgEl: false,
											styles: {
												width: '20px',
												height: '20px',
											},
										}}
									/>
								</div>
							</>
						),
						job: data?.data,
						successBtn: {
							subDetails: { method: 'PUT', url: `jobs/${id}` },
							text: 'Update Job',
						},
					}}
				/>
			)}
		</div>
	);
}
